/*
 * Translator Helper (TH)
 * Fco. Javier Domínguez Carral
 * TFC - UOC (2014)
 */
package com.ADApps.translatorhelper.contactes;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.widget.TextView;

import com.ADApps.translatorhelper.R;
import com.ADApps.translatorhelper.main.THDataBaseHelper;
import com.actionbarsherlock.app.SherlockListFragment;
import com.actionbarsherlock.view.Menu;
import com.actionbarsherlock.view.MenuInflater;
import com.actionbarsherlock.view.MenuItem;


/**
 * The Class LlistaContactesFragment.
 */
public class LlistaContactesFragment extends SherlockListFragment {
	
	/** The db. */
	private static THDataBaseHelper db;
	
	/** The list items. */
	private Contacte[] listItems;
	
	/** The Constant TIPUS_CLIENT. */
	private static final int TIPUS_CLIENT = 1;
	
	/** The Constant TIPUS_COLABORADOR. */
	private static final int TIPUS_COLABORADOR = 2;
	
	/* (non-Javadoc)
	 * @see android.support.v4.app.Fragment#onCreate(android.os.Bundle)
	 */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setHasOptionsMenu(true);
		db = new THDataBaseHelper(getActivity());
		db.open();
		
		listItems = db.getContactes().toArray(new Contacte[0]);
	}
	
	/* (non-Javadoc)
	 * @see android.support.v4.app.ListFragment#onCreateView(android.view.LayoutInflater, android.view.ViewGroup, android.os.Bundle)
	 */
	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container,
			Bundle savedInstanceState) {
		return inflater.inflate(R.layout.fragment_llista, container, false);
	}
	
	/* (non-Javadoc)
	 * @see android.support.v4.app.Fragment#onActivityCreated(android.os.Bundle)
	 */
	@Override
	public void onActivityCreated(Bundle savedInstanceState) {
		super.onActivityCreated(savedInstanceState);
		
		setListAdapter(new AdaptadorContactes(this));
		
		if (listItems.length == 0)
			((TextView) getView().findViewById(android.R.id.empty)).setText("No hi ha contactes per mostrar.");
	}
	
	/**
	 * The Class AdaptadorContactes.
	 */
	class AdaptadorContactes extends ArrayAdapter<Contacte> {
		
		/** The context. */
		Activity context;
		
		/**
		 * Instantiates a new adaptador contactes.
		 *
		 * @param context the context
		 */
		AdaptadorContactes (SherlockListFragment context) {
			super(context.getActivity(), R.layout.listitem_contacte, listItems);
			this.context = context.getActivity();
		}
		
		/* (non-Javadoc)
		 * @see android.widget.ArrayAdapter#getView(int, android.view.View, android.view.ViewGroup)
		 */
		@Override
		public View getView(int position, View convertView, ViewGroup parent) {
			LayoutInflater inflater = context.getLayoutInflater();
			View item = inflater.inflate(R.layout.listitem_contacte, null);
			
			TextView lblNomCognomsLIContacte = (TextView)item.findViewById(R.id.lblNomCognomsLIContacte);
			lblNomCognomsLIContacte.setText(listItems[position].getNomCognoms());
			
			TextView lblTipusLIContacte = (TextView)item.findViewById(R.id.lblTipusLIContacte);
			
			switch (listItems[position].getTipusContacte()) {
			case TIPUS_CLIENT:
				lblTipusLIContacte.setText(R.string.tipusContacteClient);
				break;
			case TIPUS_COLABORADOR:
				lblTipusLIContacte.setText(R.string.tipusContacteColaborador);
				break;
			default:
				break;
			}
			return item;
		}
	}

	/* (non-Javadoc)
	 * @see android.support.v4.app.ListFragment#onListItemClick(android.widget.ListView, android.view.View, int, long)
	 */
	@Override
	public void onListItemClick(ListView l, View v, int position, long id) {
		Contacte registre = (Contacte) getListAdapter().getItem(position);
		
		Intent intent = new Intent(getActivity().getBaseContext(), DetallsContacteActivity.class);
		intent.putExtra("afegir", false);
		intent.putExtra("registre", registre);
		startActivity(intent);
	}
	
	/* (non-Javadoc)
	 * @see com.actionbarsherlock.app.SherlockListFragment#onCreateOptionsMenu(android.view.Menu, android.view.MenuInflater)
	 */
	@Override
	public void onCreateOptionsMenu(Menu menu, MenuInflater inflater) {
		inflater.inflate(R.menu.llista_parametres, menu);
	}
	
	/* (non-Javadoc)
	 * @see com.actionbarsherlock.app.SherlockListFragment#onOptionsItemSelected(android.view.MenuItem)
	 */
	@Override
    public boolean onOptionsItemSelected(MenuItem item) {
		Intent intent;
		switch (item.getItemId()) {
			case R.id.menuLstParametresAfegir:
				db.close();
				intent = new Intent(getActivity().getBaseContext(), DetallsContacteActivity.class);
				intent.putExtra("afegir", true);
				startActivity(intent);
	            return true;
			default:
				return super.onOptionsItemSelected(item);
        }
    }
}
