/*
 * Translator Helper (TH)
 * Fco. Javier Domínguez Carral
 * TFC - UOC (2014)
 */
package com.ADApps.translatorhelper.factures;

import java.text.ParseException;
import java.text.SimpleDateFormat;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.widget.TextView;

import com.ADApps.translatorhelper.R;
import com.ADApps.translatorhelper.main.MainActivity;
import com.ADApps.translatorhelper.main.THDataBaseHelper;
import com.ADApps.translatorhelper.projectes.Projecte;
import com.actionbarsherlock.app.SherlockListFragment;
import com.actionbarsherlock.view.MenuItem;


/**
 * The Class LlistaFacturesFragment.
 */
public class LlistaFacturesFragment extends SherlockListFragment{
	
	/** The Constant PROVISIONAL. */
	private static final int PROVISIONAL = 1;
	
	/** The Constant PENDENT. */
	private static final int PENDENT = 2;
	
	/** The Constant COBRADA. */
	private static final int COBRADA = 3;
	
	/** The Constant FACTURA. */
	private static final int FACTURA = 3;
	
	/** The db. */
	private static THDataBaseHelper db;
	
	/** The list items. */
	private Projecte[] listItems;
	
	/* (non-Javadoc)
	 * @see android.support.v4.app.Fragment#onCreate(android.os.Bundle)
	 */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setHasOptionsMenu(true);
		db = new THDataBaseHelper(getActivity());
		db.open();
		
		try {
			listItems = db.getProjectes(FACTURA).toArray(new Projecte[0]);
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	/* (non-Javadoc)
	 * @see android.support.v4.app.ListFragment#onCreateView(android.view.LayoutInflater, android.view.ViewGroup, android.os.Bundle)
	 */
	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container,
			Bundle savedInstanceState) {
		return inflater.inflate(R.layout.fragment_llista, container, false);
	}
	
	/* (non-Javadoc)
	 * @see android.support.v4.app.Fragment#onActivityCreated(android.os.Bundle)
	 */
	@Override
	public void onActivityCreated(Bundle savedInstanceState) {
		super.onActivityCreated(savedInstanceState);
		
		setListAdapter(new AdaptadorFactures(this));
		
		if (listItems.length == 0)
			((TextView) getView().findViewById(android.R.id.empty))
					.setText("No hi ha factures per mostrar.");
	}
	
	/**
	 * The Class AdaptadorFactures.
	 */
	class AdaptadorFactures extends ArrayAdapter<Projecte> {
		
		/** The context. */
		Activity context;
		
		/**
		 * Instantiates a new adaptador factures.
		 *
		 * @param context the context
		 */
		AdaptadorFactures (SherlockListFragment context) {
			super(context.getActivity(), R.layout.listitem_factura, listItems);
			this.context = context.getActivity();
		}
		
		/* (non-Javadoc)
		 * @see android.widget.ArrayAdapter#getView(int, android.view.View, android.view.ViewGroup)
		 */
		@Override
		public View getView(int position, View convertView, ViewGroup parent) {
			LayoutInflater inflater = context.getLayoutInflater();
			View item = inflater.inflate(R.layout.listitem_factura, null);
			
			TextView tvClient = (TextView)item.findViewById(R.id.lblClientLIFactura);
			tvClient.setText(listItems[position].getClient().getNomCognoms());
			
			TextView tvEstat = (TextView)item.findViewById(R.id.lblEstatLIFactura);			
			switch (listItems[position].getFactura().getEstat()){
				case PROVISIONAL:
					tvEstat.setText(R.string.estatFac_provisional);
					break;
					
				case PENDENT:
					tvEstat.setText(R.string.estatFac_pendent);
					break;
					
				case COBRADA:
					tvEstat.setText(R.string.estatFac_cobrada);
					break;
					
				default:
					break;
			}
			
			
			TextView idFactura = (TextView)item.findViewById(R.id.lblIdFacturaLIFactura);
			idFactura.setText(listItems[position].getFactura().getIdFactura());
			
			TextView impFactura = (TextView)item.findViewById(R.id.lblImportLIFactura);
			impFactura.setText(Float.toString(listItems[position].getFactura().getImportTotal()));
			
			TextView dataFactura = (TextView)item.findViewById(R.id.lblDataLIFactura);
			SimpleDateFormat sdf = new SimpleDateFormat ("dd/MM/yyyy");
			dataFactura.setText(sdf.format(listItems[position].getFactura().getDataCobrament()));
			
			return item;
		}
	}

	/* (non-Javadoc)
	 * @see android.support.v4.app.ListFragment#onListItemClick(android.widget.ListView, android.view.View, int, long)
	 */
	@Override
	public void onListItemClick(ListView l, View v, int position, long id) {
		Projecte registre = (Projecte) getListAdapter().getItem(position);
		
		Intent intent = new Intent(getActivity().getBaseContext(), DetallsFacturaActivity.class);
		intent.putExtra("registre", registre);
		startActivity(intent);
	}

	/* (non-Javadoc)
	 * @see com.actionbarsherlock.app.SherlockListFragment#onOptionsItemSelected(android.view.MenuItem)
	 */
	@Override
    public boolean onOptionsItemSelected(MenuItem item) {
		Intent intent;
		switch (item.getItemId()) {
			case android.R.id.home:
				db.close();
	            intent = new Intent(getActivity(), MainActivity.class);
	            intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP|Intent.FLAG_ACTIVITY_NEW_TASK);
	            startActivity(intent);
	            return true;
			default:
				return super.onOptionsItemSelected(item);
        }
    }		
}
