/*
 * Translator Helper (TH)
 * Fco. Javier Domínguez Carral
 * TFC - UOC (2014)
 */
package com.ADApps.translatorhelper.main;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;

import android.annotation.SuppressLint;
import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteException;
import android.database.sqlite.SQLiteOpenHelper;

import com.ADApps.translatorhelper.contactes.Contacte;
import com.ADApps.translatorhelper.factures.Factura;
import com.ADApps.translatorhelper.parametres.conceptes.Concepte;
import com.ADApps.translatorhelper.parametres.impostos.Impost;
import com.ADApps.translatorhelper.parametres.impostos.ImpostFac;
import com.ADApps.translatorhelper.parametres.preusunitaris.PreuUnitari;
import com.ADApps.translatorhelper.parametres.preusunitaris.PreuUnitariFac;
import com.ADApps.translatorhelper.parametres.tarifes.Tarifa;
import com.ADApps.translatorhelper.parametres.variables.Variable;
import com.ADApps.translatorhelper.parametres.variables.VariableFac;
import com.ADApps.translatorhelper.projectes.Projecte;



/**
 * The Class THDataBaseHelper.
 */
public class THDataBaseHelper extends SQLiteOpenHelper {
	
	/** The Constant taulaConceptes. */
	private static final String taulaConceptes = "Conceptes";
	
	/** The Constant idConcepte. */
	private static final String idConcepte = "idConcepte";
	
	/** The Constant nomConcepte. */
	private static final String nomConcepte = "nomConcepte";
	
	/** The Constant taulaTarifes. */
	private static final String taulaTarifes = "Tarifes";
	
	/** The Constant idTarifa. */
	private static final String idTarifa = "idTarifa";
	
	/** The Constant nomTarifa. */
	private static final String nomTarifa = "nomTarifa";
	
	/** The Constant taulaPreusUnitaris. */
	private static final String taulaPreusUnitaris = "PreusUnitaris";
	
	/** The Constant idPreuUnitari. */
	private static final String idPreuUnitari = "idPreuUnitari";
	
	/** The Constant nomPreuUnitari. */
	private static final String nomPreuUnitari = "nomPreuUnitari";
	
	/** The Constant preu. */
	private static final String preu = "preu";
	
	/** The Constant unitats. */
	private static final String unitats = "unitats";
	
	/** The Constant taulaImpostos. */
	private static final String taulaImpostos = "Impostos";
	
	/** The Constant idImpost. */
	private static final String idImpost = "idImpost";
	
	/** The Constant nomImpost. */
	private static final String nomImpost = "nomImpost";
	
	/** The Constant percentatge. */
	private static final String percentatge = "percentatge";
	
	/** The Constant taulaVariables. */
	private static final String taulaVariables = "Variables";
	
	/** The Constant idVariable. */
	private static final String idVariable = "idVariable";
	
	/** The Constant nomVariable. */
	private static final String nomVariable = "nomVariable";
	
	/** The Constant valorVariable. */
	private static final String valorVariable = "valor";
	
	/** The Constant tipusOperacioVariable. */
	private static final String tipusOperacioVariable = "tipusOperacio";
	
	/** The Constant taulaContactes. */
	private static final String taulaContactes = "Contactes";
	
	/** The Constant idContacte. */
	private static final String idContacte = "idContacte";
	
	/** The Constant nomContacte. */
	private static final String nomContacte = "nomContacte";
	
	/** The Constant tipusContacte. */
	private static final String tipusContacte = "tipusContacte";
	
	/** The Constant nifCif. */
	private static final String nifCif = "nifCif";
	
	/** The Constant carrer. */
	private static final String carrer = "carrer";
	
	/** The Constant ciutat. */
	private static final String ciutat = "ciutat";
	
	/** The Constant codiPostal. */
	private static final String codiPostal = "codiPostal";
	
	/** The Constant pais. */
	private static final String pais = "pais";
	
	/** The Constant email. */
	private static final String email = "email";
	
	/** The Constant telefon. */
	private static final String telefon = "telefon";
	
	/** The Constant fax. */
	private static final String fax = "fax";
	
	/** The Constant urlWeb. */
	private static final String urlWeb = "urlWeb";
	
	/** The Constant taulaProjectes. */
	private static final String taulaProjectes = "Projectes";
	
	/** The Constant idProjecte. */
	private static final String idProjecte = "idProjecte";
	
	/** The Constant descClient. */
	private static final String descClient = "descContacte";
	
	/** The Constant nomProjecte. */
	private static final String nomProjecte = "nomProjecte";
	
	/** The Constant idFactura. */
	private static final String idFactura = "idFactura";
	
	/** The Constant dataEntrega. */
	private static final String dataEntrega = "dataEntrega";
	
	/** The Constant dataCreacio. */
	private static final String dataCreacio = "dataCreacio";
	
	/** The Constant estatProjecte. */
	private static final String estatProjecte = "estat";
	
	/** The Constant observacions. */
	private static final String observacions = "observacions";
	
	/** The Constant importProjecte. */
	private static final String importProjecte = "importProjecte";
	
	/** The Constant PRESSUPOST. */
	private static final int PRESSUPOST = 1;
	
	/** The Constant PROJECTE. */
	private static final int PROJECTE = 2;
	
	/** The Constant FACTURA. */
	private static final int FACTURA = 3;
	
	/** The Constant taulaFactures. */
	private static final String taulaFactures = "Factures";
	
	/** The Constant identificadorFactura. */
	private static final String identificadorFactura = "identificadorFactura";
	
	/** The Constant dataCobrament. */
	private static final String dataCobrament = "dataCobrament";
	
	/** The Constant estatFactura. */
	private static final String estatFactura = "estat";
	
	/** The Constant dataEstat. */
	private static final String dataEstat = "dataEstat";
	
	/** The Constant importFactura. */
	private static final String importFactura = "importTotal";
	
	/** The Constant taulaPreusUnitarisFac. */
	private static final String taulaPreusUnitarisFac = "PreusUnitarisFac";
	
	/** The Constant numUnitats. */
	private static final String numUnitats = "numUnitats";
	
	/** The Constant importPreuUnitats. */
	private static final String importPreuUnitats = "importPreuUnitats";
	
	/** The Constant taulaColaboradorsProjecte. */
	private static final String taulaColaboradorsProjecte = "ColaboradorsProjecte";
	
	/** The Constant taulaImpostosFac. */
	private static final String taulaImpostosFac = "ImpostosFac";
	
	/** The Constant importImpost. */
	private static final String importImpost = "importImpost";
	
	/** The Constant taulaVariablesFac. */
	private static final String taulaVariablesFac = "VariablesFac";
	
	/** The Constant importVariable. */
	private static final String importVariable = "importVariable";
	
	/** The db_path. */
	@SuppressLint("SdCardPath")
	private static String db_path = "/data/data/com.ADApps.translatorhelper/databases/";
	
	/** The db_name. */
	private static String db_name = "THDataBase.sqlite";
	
	/** The my data base. */
	private SQLiteDatabase myDataBase;
	
	/** The my context. */
	private final Context myContext;
	
	/** The Constant myPath. */
	private static final String myPath = db_path + db_name;
	
	/** The format. */
	SimpleDateFormat format = new SimpleDateFormat("dd/MM/yyyy");
	
	/**
	 * Instantiates a new TH data base helper.
	 *
	 * @param context the context
	 */
	public THDataBaseHelper (Context context) {
		super(context, db_name, null, 1);
		this.myContext = context;
	}
	
	/* Creates an empty DB and fills it with our db file.*/
	/**
	 * Creates the data base.
	 *
	 * @throws IOException Signals that an I/O exception has occurred.
	 */
	public void createDataBase() throws IOException {
		boolean dbExists = checkDataBase();
		
		// If the DB doesn't exist create empty data base at default path and refill with our db.
		if (!dbExists) {
			this.getReadableDatabase();
			
			try {
				copyDataBase();
				
			}catch(IOException e){
				throw new Error("Error copiant la Base de Dades.");
			}
		}
	}
	
	// Checks if the DB exists.
	/**
	 * Check data base.
	 *
	 * @return true, if successful
	 */
	private boolean checkDataBase() {
		SQLiteDatabase checkDB = null;
		
		try {
			checkDB = SQLiteDatabase.openDatabase(myPath, null, SQLiteDatabase.OPEN_READONLY);
		}catch (SQLiteException e){
			// The DB doesn't exists yet.
		}
		
		if (checkDB != null) {
			checkDB.close();
			return true;
		}else return false;
		// return checkDB != null ? true : false;
	}
	
	/* Copies our db from assets folder to the new DB at the system folder.*/
	/**
	 * Copy data base.
	 *
	 * @throws IOException Signals that an I/O exception has occurred.
	 */
	private void copyDataBase() throws IOException{
		//Open the input file.
		InputStream myInput = myContext.getAssets().open(db_name);
		
		//Path to the new DB.
		String outFileName = db_path + db_name;
		
		//Open the new DB as output.
		OutputStream myOutput = new FileOutputStream(outFileName);
		
		//Transfer the bytes from input to output.
		byte[] buffer = new byte[1024];
		int length;
		while ((length = myInput.read(buffer)) > 0) {
			myOutput.write(buffer, 0, length);
		}
		
		myOutput.flush();
		myOutput.close();
		myInput.close();
	}

	// Opens the DB.
	/**
	 * Open.
	 *
	 * @throws SQLException the SQL exception
	 */
	public void open() throws SQLException{
		try{
			createDataBase();
		}catch(IOException e){
			throw new Error("No ha estat possible crear la Base de dades.");
		}
		
		myDataBase = SQLiteDatabase.openDatabase(myPath, null, SQLiteDatabase.OPEN_READWRITE);
	}
	
	/* (non-Javadoc)
	 * @see android.database.sqlite.SQLiteOpenHelper#close()
	 */
	@Override
	public synchronized void close() {
		if (myDataBase != null) {
			myDataBase.close();
			super.close();
		}
	}

	/* (non-Javadoc)
	 * @see android.database.sqlite.SQLiteOpenHelper#onCreate(android.database.sqlite.SQLiteDatabase)
	 */
	@Override
	public void onCreate(SQLiteDatabase arg0) {
		
	}

	/* (non-Javadoc)
	 * @see android.database.sqlite.SQLiteOpenHelper#onUpgrade(android.database.sqlite.SQLiteDatabase, int, int)
	 */
	@Override
	public void onUpgrade(SQLiteDatabase arg0, int arg1, int arg2) {
		
	}	
	
	
	/*
	 * CONCEPTES
	 * */
	
	/**
	 * Gets the concepte.
	 *
	 * @param id the id
	 * @return the concepte
	 */
	public Concepte getConcepte (int id) {
		Concepte concepte;
		String[] columnes = new String[] {idConcepte, nomConcepte};
		
		Cursor resultat = 
				myDataBase.query(taulaConceptes, columnes, idConcepte + " = " + id, null, null, null, null);
		
		if (resultat.moveToFirst()) {
			concepte = new Concepte(
					resultat.getInt(resultat.getColumnIndex(idConcepte)),
					resultat.getString(resultat.getColumnIndex(nomConcepte)));
			return concepte;
		}else {
			return null;
		}
	}
	
	/**
	 * Gets the conceptes.
	 *
	 * @return the conceptes
	 */
	public ArrayList<Concepte> getConceptes() {
		ArrayList<Concepte> conceptes = new ArrayList<Concepte>();
		String[]  columnes = new String[] {idConcepte, nomConcepte};
		Cursor resultat = 
				myDataBase.query(taulaConceptes, columnes, null, null, null, null, nomConcepte);
		
		if (resultat.moveToFirst()) {
			do {
				conceptes.add(new Concepte(
					resultat.getInt(resultat.getColumnIndex(idConcepte)),
					resultat.getString(resultat.getColumnIndex(nomConcepte))));
			} while(resultat.moveToNext());
		}
		return conceptes;
	}
	
	/**
	 * Insert concepte.
	 *
	 * @param nom the nom
	 * @return the boolean
	 */
	public Boolean insertConcepte (String nom) {
		String[] args = new String[]{nom};		
		myDataBase.execSQL("INSERT INTO " + taulaConceptes +" (" + nomConcepte + ") VALUES (?) ", args);
		return true;
	}
	
	/**
	 * Update concepte.
	 *
	 * @param id the id
	 * @param nom the nom
	 * @return the boolean
	 */
	public Boolean updateConcepte (int id, String nom) {
		ContentValues valors = new ContentValues();
		valors.put(nomConcepte, nom);
		 
		String[] args = new String[]{Integer.toString(id)};
		myDataBase.update(taulaConceptes, valors, "idConcepte=?", args);
		return true;
	}
	
	/**
	 * Delete concepte.
	 *
	 * @param id the id
	 * @return the boolean
	 */
	public Boolean deleteConcepte (int id) {
		String[] args = new String[]{Integer.toString(id)};
		myDataBase.delete(taulaConceptes, "idConcepte=?", args);
		return true;
	}
	
	/**
	 * Gets the conceptes spinner.
	 *
	 * @return the conceptes spinner
	 */
	public ArrayList<String> getConceptesSpinner () {
		ArrayList<String> conceptesSP = new ArrayList<String>();
		String[] columnes = new String[] {nomConcepte};
		
		Cursor resultat = myDataBase.query(taulaConceptes, columnes, null, null, null, null, nomConcepte);
		
		if (resultat.moveToFirst()) {
			do{
				conceptesSP.add(resultat.getString(resultat.getColumnIndex(nomConcepte)));
			}while (resultat.moveToNext());
		}
		return conceptesSP;
	}
	
	/**
	 * Gets the ID concepte.
	 *
	 * @param nom the nom
	 * @return the ID concepte
	 */
	public int getIDConcepte (String nom) {
		int id;
		String[] columnes = new String[] {idConcepte};
		
		Cursor resultat = 
				myDataBase.query(taulaConceptes, columnes, nomConcepte + " = '" + nom + "'", null, null, null, null);
		
		if (resultat.moveToFirst()) {
			id = resultat.getInt(resultat.getColumnIndex(idConcepte));
			return id;
		}else {
			return -1;
		}
	}
	
	/**
	 * Checks if is concepte referenced.
	 *
	 * @param id the id
	 * @return true, if is concepte referenced
	 */
	public boolean isConcepteReferenced (int id) {
		String[] columnes = new String[] {idConcepte};
		Cursor resultat = myDataBase.query(taulaTarifes, columnes, idConcepte + " = " + id, null, null, null, null);
		return (!resultat.moveToFirst());
	}
	
	
	
	/*
	 * TARIFES
	 * */
	
	/**
	 * Gets the tarifa.
	 *
	 * @param id the id
	 * @return the tarifa
	 */
	public Tarifa getTarifa (int id) {
		Tarifa tarifa;
		Concepte concepte;
		String[] columnes = new String[] {idTarifa, nomTarifa, idConcepte};
		
		Cursor resultat = 
				myDataBase.query(taulaTarifes, columnes, idTarifa + " = " + id, null, null, null, null); 
		
		if (resultat.moveToFirst()) {
			concepte = getConcepte(resultat.getInt(resultat.getColumnIndex(idConcepte)));
			
			tarifa = new Tarifa(
					resultat.getInt(resultat.getColumnIndex(idTarifa)),
					resultat.getString(resultat.getColumnIndex(nomTarifa)),
					concepte);
			return tarifa;
		}else {
			return null;
		}
	}
	
	/**
	 * Gets the tarifa.
	 *
	 * @param nomTar the nom tar
	 * @param idC the id c
	 * @return the tarifa
	 */
	public Tarifa getTarifa (String nomTar, int idC) {
		Tarifa tarifa;
		Concepte concepte;
		String[] columnes = new String[] {idTarifa, nomTarifa, idConcepte};
		
		Cursor resultat = 
				myDataBase.query(taulaTarifes, columnes, idConcepte + " = " + idC + " AND " + nomTarifa + " = '" + nomTar + "'", null, null, null, null);
				
		if (resultat.moveToFirst()) {
			concepte = getConcepte(resultat.getInt(resultat.getColumnIndex(idConcepte)));
			
			tarifa = new Tarifa(
					resultat.getInt(resultat.getColumnIndex(idTarifa)),
					resultat.getString(resultat.getColumnIndex(nomTarifa)),
					concepte);
			return tarifa;
		}else {
			return null;
		}
	}
	
	/**
	 * Gets the tarifes.
	 *
	 * @return the tarifes
	 */
	public ArrayList<Tarifa> getTarifes() {
		Concepte concepte;
		ArrayList<Tarifa> tarifes = new ArrayList<Tarifa>();
		String[] columnes = new String[] {idTarifa, nomTarifa, idConcepte};
		Cursor resultat = 
				myDataBase.query(taulaTarifes, columnes, null, null, null, null, nomTarifa);
		
		if (resultat.moveToFirst()) {
			do {
				concepte = getConcepte(resultat.getInt(resultat.getColumnIndex(idConcepte)));
				
				tarifes.add(new Tarifa(
						resultat.getInt(resultat.getColumnIndex(idTarifa)),
						resultat.getString(resultat.getColumnIndex(nomTarifa)),
						concepte));
			} while(resultat.moveToNext());
		}
		return tarifes;
	}
	
	/**
	 * Insert tarifa.
	 *
	 * @param nom the nom
	 * @param idC the id c
	 * @return the boolean
	 */
	public Boolean insertTarifa (String nom, int idC) {
		String[] args = new String[]{nom, Integer.toString(idC)};		
		myDataBase.execSQL("INSERT INTO " + taulaTarifes +" (" + nomTarifa + ", " + idConcepte 
				+ ") VALUES (?,?) ", args);
		return true;
	}
	
	/**
	 * Update tarifa.
	 *
	 * @param id the id
	 * @param nom the nom
	 * @param idC the id c
	 * @return the boolean
	 */
	public Boolean updateTarifa (int id, String nom, int idC) {
		ContentValues valors = new ContentValues();
		valors.put(nomTarifa, nom);
		valors.put(idConcepte, idC);
		 
		String[] args = new String[]{Integer.toString(id)};
		myDataBase.update(taulaTarifes, valors, "idTarifa=?", args);
		return true;
	}
	
	/**
	 * Delete tarifa.
	 *
	 * @param id the id
	 * @return the boolean
	 */
	public Boolean deleteTarifa (int id) {
		String[] args = new String[]{Integer.toString(id)};
		myDataBase.delete(taulaTarifes, "idTarifa=?", args);
		return true;
	}
	
	/**
	 * Gets the tarifes spinner.
	 *
	 * @param idC the id c
	 * @return the tarifes spinner
	 */
	public ArrayList<String> getTarifesSpinner (int idC) {
		ArrayList<String> tarifesSP = new ArrayList<String>();
		String[] columnes = new String[] {nomTarifa};
		
		Cursor resultat = myDataBase.query(taulaTarifes, columnes, idConcepte + " = " + idC, null, null, null, nomTarifa);
		
		if (resultat.moveToFirst()) {
			do{
				tarifesSP.add(resultat.getString(resultat.getColumnIndex(nomTarifa)));
			}while (resultat.moveToNext());
		}
		return tarifesSP;
	}
		
	/**
	 * Gets the ID tarifa.
	 *
	 * @param nom the nom
	 * @param idC the id c
	 * @return the ID tarifa
	 */
	public int getIDTarifa (String nom, int idC) {
		int id;
		String[] columnes = new String[] {idTarifa};
		
		Cursor resultat = myDataBase.query(taulaTarifes, columnes, nomTarifa + " = '" + nom + "' AND " 
				+ idConcepte + " = " + idC, null, null, null, null);
		
		if (resultat.moveToFirst()) {
			id = resultat.getInt(resultat.getColumnIndex(idTarifa));
			return id;
		}else {
			return -1;
		}
	}
	
	/**
	 * Checks if is tarifa referenced.
	 *
	 * @param id the id
	 * @return true, if is tarifa referenced
	 */
	public boolean isTarifaReferenced (int id) {
		String[] columnes = new String[] {idTarifa};
		Cursor resultat = 
				myDataBase.query(taulaPreusUnitaris, columnes, idTarifa + " = " + id, null, null, null, null);
		return (!resultat.moveToFirst());
	}
	
	
	/*
	 * PREUS UNITARIS
	 * */
	
	/**
	 * Gets the preu unitari.
	 *
	 * @param id the id
	 * @return the preu unitari
	 */
	public PreuUnitari getPreuUnitari (int id) {
		PreuUnitari preuUnitari;
		Tarifa tarifa;
		String[] columnes = new String[] {idPreuUnitari, idTarifa, nomPreuUnitari, preu, unitats};
		
		Cursor resultat = 
				myDataBase.query(taulaPreusUnitaris, columnes, idPreuUnitari + " = " + id, null, null, null, null);
		
		if (resultat.moveToFirst()) {
			tarifa = getTarifa(resultat.getInt(resultat.getColumnIndex(idTarifa)));
			
			preuUnitari = new PreuUnitari(
					resultat.getInt(resultat.getColumnIndex(idPreuUnitari)),
					tarifa,
					resultat.getString(resultat.getColumnIndex(nomPreuUnitari)),
					Float.parseFloat(resultat.getString(resultat.getColumnIndex(preu))),
					resultat.getString(resultat.getColumnIndex(unitats)));
			return preuUnitari;
		}else {
			return null;
		}
	}
	
	/**
	 * Gets the preu unitari.
	 *
	 * @param idT the id t
	 * @param nomPU the nom pu
	 * @return the preu unitari
	 */
	public PreuUnitari getPreuUnitari (int idT, String nomPU) {
		PreuUnitari preuUnitari;
		String[] columnes = new String[] {idPreuUnitari, idTarifa, nomPreuUnitari, preu, unitats};
		
		Cursor resultat = 
				myDataBase.query(taulaPreusUnitaris, columnes, idTarifa + " = " + idT + " AND " + nomPreuUnitari + " = '" + nomPU + "'", 
						null, null, null, null);
		
		if (resultat.moveToFirst()) {
			preuUnitari = new PreuUnitari(
					resultat.getInt(resultat.getColumnIndex(idPreuUnitari)),
					getTarifa(idT),
					resultat.getString(resultat.getColumnIndex(nomPreuUnitari)),
					Float.parseFloat(resultat.getString(resultat.getColumnIndex(preu))),
					resultat.getString(resultat.getColumnIndex(unitats)));
			return preuUnitari;
		}else {
			return null;
		}
	}
	
	/**
	 * Gets the preus unitaris.
	 *
	 * @return the preus unitaris
	 */
	public ArrayList<PreuUnitari> getPreusUnitaris() {
		Tarifa tarifa;
		ArrayList<PreuUnitari> preuUnitari = new ArrayList<PreuUnitari>();
		String[] columnes = new String[] {idPreuUnitari, idTarifa, nomPreuUnitari, preu, unitats};
		Cursor resultat = 
				myDataBase.query(taulaPreusUnitaris, columnes, null, null, null, null, nomPreuUnitari);
				
		if (resultat.moveToFirst()) {
			do {
				tarifa = getTarifa(resultat.getInt(resultat.getColumnIndex(idTarifa)));
				
				preuUnitari.add(new PreuUnitari(
						resultat.getInt(resultat.getColumnIndex(idPreuUnitari)),
						tarifa,
						resultat.getString(resultat.getColumnIndex(nomPreuUnitari)),
						Float.parseFloat(resultat.getString(resultat.getColumnIndex(preu))),
						resultat.getString(resultat.getColumnIndex(unitats))));
			} while(resultat.moveToNext());
		}
		
		return preuUnitari;
	}
	
	/**
	 * Insert preu unitari.
	 *
	 * @param idTar the id tar
	 * @param nomPU the nom pu
	 * @param pre the pre
	 * @param uni the uni
	 * @return the boolean
	 */
	public Boolean insertPreuUnitari (int idTar, String nomPU, String pre, String uni) {
		String[] args = new String[]{Integer.toString(idTar), nomPU, pre, uni};		
		myDataBase.execSQL("INSERT INTO " + taulaPreusUnitaris +" (" + idTarifa + ", " + nomPreuUnitari + ", " 
				+ preu + ", " + unitats + ") VALUES (?,?,?,?) ", args);
		return true;
	}
	
	/**
	 * Update preu unitari.
	 *
	 * @param idPre the id pre
	 * @param idT the id t
	 * @param nomPre the nom pre
	 * @param pre the pre
	 * @param uni the uni
	 * @return the boolean
	 */
	public Boolean updatePreuUnitari (int idPre, int idT, String nomPre, String pre, String uni) {
		ContentValues valors = new ContentValues();
		valors.put(idTarifa, idT);
		valors.put(nomPreuUnitari, nomPre);
		valors.put(preu, pre);
		valors.put(unitats, uni);
		 
		String[] args = new String[]{Integer.toString(idPre)};
		myDataBase.update(taulaPreusUnitaris, valors, "idPreuUnitari = ?", args);
		return true;
	}
	
	/**
	 * Delete preu unitari.
	 *
	 * @param id the id
	 * @return the boolean
	 */
	public Boolean deletePreuUnitari (int id) {
		String[] args = new String[]{Integer.toString(id)};
		myDataBase.delete(taulaPreusUnitaris, "idPreuUnitari = ?", args);
		return true;
	}
	
	/**
	 * Gets the preus unitaris spinner.
	 *
	 * @param idT the id t
	 * @return the preus unitaris spinner
	 */
	public ArrayList<String> getPreusUnitarisSpinner (int idT) {
		ArrayList<String> preusUnitarisSP = new ArrayList<String>();
		String[] columnes = new String[] {nomPreuUnitari};
		
		Cursor resultat = myDataBase.query(taulaPreusUnitaris, columnes, idTarifa + " = " + idT, null, null, null, nomPreuUnitari);
		
		if (resultat.moveToFirst()) {
			do{
				preusUnitarisSP.add(resultat.getString(resultat.getColumnIndex(nomPreuUnitari)));
			}while (resultat.moveToNext());
		}
		return preusUnitarisSP;
	}
	
	/**
	 * Gets the ID preu unitari.
	 *
	 * @param idTar the id tar
	 * @param nom the nom
	 * @return the ID preu unitari
	 */
	public int getIDPreuUnitari (int idTar, String nom) {
		String[] columnes = new String[] {idPreuUnitari};
		
		Cursor resultat = myDataBase.query(taulaPreusUnitaris, columnes, idTarifa + " = " + idTar + " AND " 
				+ nomPreuUnitari + " = '" + nom + "'", null, null, null, null);
		
		if (resultat.moveToFirst()) {
			return resultat.getInt(resultat.getColumnIndex(idPreuUnitari));
		}else {
			return -1;
		}
	}
	
	/**
	 * Gets the ID preu unitari.
	 *
	 * @param idTar the id tar
	 * @param nom the nom
	 * @param pre the pre
	 * @param uni the uni
	 * @return the ID preu unitari
	 */
	public int getIDPreuUnitari (int idTar, String nom, String pre, String uni) {
		String[] columnes = new String[] {idPreuUnitari};
		
		Cursor resultat = myDataBase.query(taulaPreusUnitaris, columnes, idTarifa + " = " + idTar + " AND " 
				+ nomPreuUnitari + " = '" + nom + "' AND " + preu + " = " + pre + " AND " + unitats + " = '" + uni 
				+ "'", null, null, null, null);
		
		if (resultat.moveToFirst()) {
			return resultat.getInt(resultat.getColumnIndex(idPreuUnitari));
		}else {
			return -1;
		}
	}

	
	/*
	* IMPOSTOS
	**/
	
	/**
	 * Gets the impost.
	 *
	 * @param id the id
	 * @return the impost
	 */
	public Impost getImpost (int id) {
		Impost impost;
		String[] columnes = new String[] {idImpost, nomImpost, percentatge};
		
		Cursor resultat = 
				myDataBase.query(taulaImpostos, columnes, idImpost + " = " + id, null, null, null, null);
		
		if (resultat.moveToFirst()) {
			impost = new Impost(
					resultat.getInt(resultat.getColumnIndex(idImpost)),
					resultat.getString(resultat.getColumnIndex(nomImpost)),
					resultat.getFloat(resultat.getColumnIndex(percentatge)));
			return impost;
		}else {
			return null;
		}
	}
	
	/**
	 * Gets the impostos.
	 *
	 * @return the impostos
	 */
	public ArrayList<Impost> getImpostos() {
		ArrayList<Impost> impostos = new ArrayList<Impost>();
		String[] columnes = new String[] {idImpost, nomImpost, percentatge};
		Cursor resultat = 
				myDataBase.query(taulaImpostos, columnes, null, null, null, null, nomImpost);
		
		if (resultat.moveToFirst()) {
			do {
				impostos.add(new Impost(
						resultat.getInt(resultat.getColumnIndex(idImpost)),
						resultat.getString(resultat.getColumnIndex(nomImpost)),
						resultat.getFloat(resultat.getColumnIndex(percentatge))));
			} while(resultat.moveToNext());
		}
		
		return impostos;
	}
	
	/**
	 * Insert impost.
	 *
	 * @param nomImp the nom imp
	 * @param percent the percent
	 * @return the boolean
	 */
	public Boolean insertImpost (String nomImp, String percent) {
		String[] args = new String[]{nomImp, percent};		
		myDataBase.execSQL("INSERT INTO " + taulaImpostos +" (" + nomImpost + ", " + percentatge + ") VALUES (?,?)", args);
		return true;
	}
	
	/**
	 * Update impost.
	 *
	 * @param idImp the id imp
	 * @param nomImp the nom imp
	 * @param percent the percent
	 * @return the boolean
	 */
	public Boolean updateImpost (int idImp, String nomImp, String percent) {
		ContentValues valors = new ContentValues();
		valors.put(nomImpost, nomImp);
		valors.put(percentatge, percent);
		 
		String[] args = new String[]{Integer.toString(idImp)};
		myDataBase.update(taulaImpostos, valors, "idImpost = ?", args);
		return true;
	}
	
	/**
	 * Delete impost.
	 *
	 * @param id the id
	 * @return the boolean
	 */
	public Boolean deleteImpost (int id) {
		String[] args = new String[]{Integer.toString(id)};
		myDataBase.delete(taulaImpostos, "idImpost = ?", args);
		return true;
	}

	/**
	 * Gets the impostos spinner.
	 *
	 * @return the impostos spinner
	 */
	public ArrayList<String> getImpostosSpinner () {
		ArrayList<String> impostosSP = new ArrayList<String>();
		String[] columnes = new String[] {nomImpost};
		
		Cursor resultat = myDataBase.query(taulaImpostos, columnes, null, null, null, null, nomImpost);
		
		if (resultat.moveToFirst()) {
			do{
				impostosSP.add(resultat.getString(resultat.getColumnIndex(nomImpost)));
			}while (resultat.moveToNext());
		}
		return impostosSP;
	}
	
	/**
	 * Gets the ID impost.
	 *
	 * @param nom the nom
	 * @return the ID impost
	 */
	public int getIDImpost (String nom) {
		int id;
		String[] columnes = new String[] {idImpost};
		
		Cursor resultat = myDataBase.query(taulaImpostos, columnes, nomImpost + " = '" + nom 
				+ "'", null, null, null, null);
		
		if (resultat.moveToFirst()) {
			id = resultat.getInt(resultat.getColumnIndex(idImpost));
			return id;
		}else {
			return -1;
		}
	}
	
	/**
	 * Gets the ID impost.
	 *
	 * @param nom the nom
	 * @param percent the percent
	 * @return the ID impost
	 */
	public int getIDImpost (String nom, String percent) {
		int id;
		String[] columnes = new String[] {idImpost};
		
		Cursor resultat = myDataBase.query(taulaImpostos, columnes, nomImpost + " = '" + nom + "' AND " 
				+ percentatge + " = " + percent, null, null, null, null);
		
		if (resultat.moveToFirst()) {
			id = resultat.getInt(resultat.getColumnIndex(idImpost));
			return id;
		}else {
			return -1;
		}
	}


	/*
	* VARIABLES
	**/
	
	/**
	 * Gets the variable.
	 *
	 * @param id the id
	 * @return the variable
	 */
	public Variable getVariable (int id) {
		Variable variable;
		String[] columnes = new String[] {idVariable, nomVariable, valorVariable, tipusOperacioVariable};
		
		Cursor resultat = 
				myDataBase.query(taulaVariables, columnes, idVariable + " = " + id, null, null, null, null);
		
		if (resultat.moveToFirst()) {
			variable = new Variable(
					resultat.getInt(resultat.getColumnIndex(idVariable)),
					resultat.getString(resultat.getColumnIndex(nomVariable)),
					Float.parseFloat(resultat.getString(resultat.getColumnIndex(valorVariable))),
					resultat.getInt(resultat.getColumnIndex(tipusOperacioVariable)));
			return variable;
		}else {
			return null;
		}
	}
	
	/**
	 * Gets the variables.
	 *
	 * @return the variables
	 */
	public ArrayList<Variable> getVariables() {
		ArrayList<Variable> variables = new ArrayList<Variable>();
		String[] columnes = new String[] {idVariable, nomVariable, valorVariable, tipusOperacioVariable};
		Cursor resultat = 
				myDataBase.query(taulaVariables, columnes, null, null, null, null, nomVariable);
		
		if (resultat.moveToFirst()) {
			do {
				variables.add(new Variable(
						resultat.getInt(resultat.getColumnIndex(idVariable)),
						resultat.getString(resultat.getColumnIndex(nomVariable)),
						Float.parseFloat(resultat.getString(resultat.getColumnIndex(valorVariable))),
						resultat.getInt(resultat.getColumnIndex(tipusOperacioVariable))));
			} while(resultat.moveToNext());
		}
		
		return variables;
	}
	
	/**
	 * Insert variable.
	 *
	 * @param nomVar the nom var
	 * @param vVar the v var
	 * @param tipus the tipus
	 * @return the boolean
	 */
	public Boolean insertVariable (String nomVar, String vVar, int tipus) {
		String[] args = new String[]{nomVar, vVar, Integer.toString(tipus)};		
		myDataBase.execSQL("INSERT INTO " + taulaVariables +" (" + nomVariable + ", " + valorVariable + ", " 
				+ tipusOperacioVariable + ") VALUES (?,?,?) ", args);
		return true;
	}
	
	/**
	 * Update variable.
	 *
	 * @param idVar the id var
	 * @param nomVar the nom var
	 * @param vVar the v var
	 * @param tipus the tipus
	 * @return the boolean
	 */
	public Boolean updateVariable (int idVar, String nomVar, String vVar, int tipus) {
		ContentValues valors = new ContentValues();
		valors.put(nomVariable, nomVar);
		valors.put(valorVariable, vVar);
		valors.put(tipusOperacioVariable, tipus);
		 
		String[] args = new String[]{Integer.toString(idVar)};
		myDataBase.update(taulaVariables, valors, "idVariable = ?", args);
		return true;
	}
	
	/**
	 * Delete variable.
	 *
	 * @param id the id
	 * @return the boolean
	 */
	public Boolean deleteVariable (int id) {
		String[] args = new String[]{Integer.toString(id)};
		myDataBase.delete(taulaVariables, "idVariable = ?", args);
		return true;
	}

	/**
	 * Gets the variables spinner.
	 *
	 * @return the variables spinner
	 */
	public ArrayList<String> getVariablesSpinner () {
		ArrayList<String> variablesSP = new ArrayList<String>();
		String[] columnes = new String[] {nomVariable};
		
		Cursor resultat = myDataBase.query(taulaVariables, columnes, null, null, null, null, nomVariable);
		
		if (resultat.moveToFirst()) {
			do{
				variablesSP.add(resultat.getString(resultat.getColumnIndex(nomVariable)));
			}while (resultat.moveToNext());
		}
		return variablesSP;
	}
	
	/**
	 * Gets the ID variable.
	 *
	 * @param nom the nom
	 * @return the ID variable
	 */
	public int getIDVariable (String nom) {
		int id;
		String[] columnes = new String[] {idVariable};
		
		Cursor resultat = myDataBase.query(taulaVariables, columnes, nomVariable + " = '" + nom 
				+ "'", null, null, null, null);
		
		if (resultat.moveToFirst()) {
			id = resultat.getInt(resultat.getColumnIndex(idVariable));
			return id;
		}else {
			return -1;
		}
	}
	
	/**
	 * Gets the ID variable.
	 *
	 * @param nom the nom
	 * @param vVar the v var
	 * @param tipus the tipus
	 * @return the ID variable
	 */
	public int getIDVariable (String nom, String vVar, int tipus) {
		int id;
		String[] columnes = new String[] {idVariable};
		
		Cursor resultat = myDataBase.query(taulaVariables, columnes, nomVariable + " = '" + nom + "' AND " 
				+ valorVariable + " = " + vVar + " AND " + tipusOperacioVariable + " = " + tipus, null, null
				, null, null);
		
		if (resultat.moveToFirst()) {
			id = resultat.getInt(resultat.getColumnIndex(idVariable));
			return id;
		}else {
			return -1;
		}
	}
	
	
	/*
	* CONTACTES
	**/
	
	/**
	 * Gets the contacte.
	 *
	 * @param id the id
	 * @return the contacte
	 */
	public Contacte getContacte (int id) {
		Contacte contacte;
		String[] columnes = new String[] {idContacte, nomContacte, tipusContacte, nifCif, carrer, ciutat, 
				codiPostal, pais, email, telefon, fax, urlWeb};
		
		Cursor resultat = 
				myDataBase.query(taulaContactes, columnes, idContacte + " = " + id, null, null, null, null);
		
		if (resultat.moveToFirst()) {
			contacte = new Contacte(
					resultat.getInt(resultat.getColumnIndex(idContacte)),
					resultat.getString(resultat.getColumnIndex(nomContacte)),
					resultat.getInt(resultat.getColumnIndex(tipusContacte)),
					resultat.getString(resultat.getColumnIndex(nifCif)),
					resultat.getString(resultat.getColumnIndex(carrer)),
					resultat.getString(resultat.getColumnIndex(ciutat)),
					resultat.getString(resultat.getColumnIndex(codiPostal)),
					resultat.getString(resultat.getColumnIndex(pais)),
					resultat.getString(resultat.getColumnIndex(email)),
					resultat.getString(resultat.getColumnIndex(telefon)),
					resultat.getString(resultat.getColumnIndex(fax)),
					resultat.getString(resultat.getColumnIndex(urlWeb)));
			return contacte;
		}else {
			return null;
		}
	}
	
	/**
	 * Gets the contactes.
	 *
	 * @return the contactes
	 */
	public ArrayList<Contacte> getContactes() {
		ArrayList<Contacte> contactes = new ArrayList<Contacte>();
		String[] columnes = new String[] {idContacte, nomContacte, tipusContacte, nifCif, carrer, ciutat, 
				codiPostal, pais, email, telefon, fax, urlWeb};
		
		Cursor resultat = myDataBase.query(taulaContactes, columnes, null, null, null, null, nomContacte);		
		
		if (resultat.moveToFirst()) {
			do {
				contactes.add(new Contacte(
						resultat.getInt(resultat.getColumnIndex(idContacte)),
						resultat.getString(resultat.getColumnIndex(nomContacte)),
						resultat.getInt(resultat.getColumnIndex(tipusContacte)),
						resultat.getString(resultat.getColumnIndex(nifCif)),
						resultat.getString(resultat.getColumnIndex(carrer)),
						resultat.getString(resultat.getColumnIndex(ciutat)),
						resultat.getString(resultat.getColumnIndex(codiPostal)),
						resultat.getString(resultat.getColumnIndex(pais)),
						resultat.getString(resultat.getColumnIndex(email)),
						resultat.getString(resultat.getColumnIndex(telefon)),
						resultat.getString(resultat.getColumnIndex(fax)),
						resultat.getString(resultat.getColumnIndex(urlWeb))));
			} while(resultat.moveToNext());
		}
		return contactes;
	}
	
	/**
	 * Gets the contactes dialeg.
	 *
	 * @param isClient the is client
	 * @return the contactes dialeg
	 */
	public ArrayList<String> getContactesDialeg(boolean isClient) {
		ArrayList<String> contactes = new ArrayList<String>();
		String[] columnes = new String[] {nomContacte};
		Cursor resultat;
		if (isClient){
			resultat = myDataBase.query(taulaContactes, columnes, tipusContacte + " = 1", null, null, null, nomContacte);
		}else{
			resultat = myDataBase.query(taulaContactes, columnes, tipusContacte + " = 2", null, null, null, nomContacte);
		}
		
		if (resultat.moveToFirst()) {
			do {
				
				contactes.add(resultat.getString(resultat.getColumnIndex(nomContacte)));
				
			} while(resultat.moveToNext());
		}
		return contactes;
	}
	
	/**
	 * Gets the colaboradors.
	 *
	 * @param projecte the projecte
	 * @return the colaboradors
	 */
	public ArrayList<Contacte> getColaboradors(int projecte) {
		String[] columnes = new String[] {idContacte};
		Cursor resultat = 
				myDataBase.query(taulaColaboradorsProjecte, columnes, idProjecte + " = " + projecte, null, null, null, null);
		
		ArrayList<Contacte> colaboradors = new ArrayList<Contacte>();
				
		if (resultat.moveToFirst()) {
			do {
				colaboradors.add(getContacte(
						resultat.getInt(resultat.getColumnIndex(idContacte))));
			} while(resultat.moveToNext());
		}
		return colaboradors;
	}
	
	/**
	 * Insert contacte.
	 *
	 * @param nomCon the nom con
	 * @param tipusCon the tipus con
	 * @param nif the nif
	 * @param carrerCon the carrer con
	 * @param ciutatCon the ciutat con
	 * @param codiPostalCon the codi postal con
	 * @param paisCon the pais con
	 * @param emailCon the email con
	 * @param telefonCon the telefon con
	 * @param faxCon the fax con
	 * @param urlCon the url con
	 * @return the boolean
	 */
	public Boolean insertContacte (String nomCon, int tipusCon, String nif, String carrerCon, String ciutatCon,
			String codiPostalCon, String paisCon, String emailCon, String telefonCon, String faxCon, String urlCon) {
		String[] args = new String[] {nomCon, Integer.toString(tipusCon), nif, carrerCon, ciutatCon, codiPostalCon
				, paisCon, emailCon, telefonCon, faxCon, urlCon};		
		myDataBase.execSQL("INSERT INTO " + taulaContactes +" (" 
				+ nomContacte + ", " 
				+ tipusContacte + ", "
				+ nifCif + ", " 
				+ carrer + ", "
				+ ciutat + ", " 
				+ codiPostal + ", "
				+ pais + ", " 
				+ email + ", "
				+ telefon + ", " 
				+ fax + ", "
				+ urlWeb + ") VALUES (?,?,?,?,?,?,?,?,?,?,?) ", args);
		return true;
	}
	
	/**
	 * Insert colaborador.
	 *
	 * @param idPro the id pro
	 * @param idCont the id cont
	 * @return the boolean
	 */
	public Boolean insertColaborador (int idPro, int idCont) {
		String[] args = new String[] {Integer.toString(idPro), Integer.toString(idCont)};		
		myDataBase.execSQL("INSERT INTO " + taulaColaboradorsProjecte +" (" + idProjecte + ", " + idContacte 
				+ ") VALUES (?,?) ", args);
		return true;
	}
	
	/**
	 * Update contacte.
	 *
	 * @param idCon the id con
	 * @param nomCon the nom con
	 * @param tipusCon the tipus con
	 * @param nif the nif
	 * @param carrerCon the carrer con
	 * @param ciutatCon the ciutat con
	 * @param codiPostalCon the codi postal con
	 * @param paisCon the pais con
	 * @param emailCon the email con
	 * @param telefonCon the telefon con
	 * @param faxCon the fax con
	 * @param urlCon the url con
	 * @return the boolean
	 */
	public Boolean updateContacte (int idCon, String nomCon, int tipusCon, String nif, String carrerCon, 
			String ciutatCon, String codiPostalCon, String paisCon, String emailCon, String telefonCon, String faxCon, 
			String urlCon) {
		ContentValues valors = new ContentValues();
		valors.put(nomContacte, nomCon);
		valors.put(tipusContacte, tipusCon);
		valors.put(nifCif, nif);
		valors.put(carrer, carrerCon);
		valors.put(ciutat, ciutatCon);
		valors.put(codiPostal, codiPostalCon);
		valors.put(pais, paisCon);
		valors.put(email, emailCon);
		valors.put(telefon, telefonCon);
		valors.put(fax, faxCon);
		valors.put(urlWeb, urlCon);
		 
		String[] args = new String[]{Integer.toString(idCon)};
		myDataBase.update(taulaContactes, valors, "idContacte = ?", args);
		return true;
	}
	
	/**
	 * Delete colaboradors.
	 *
	 * @param id the id
	 * @return the boolean
	 */
	public Boolean deleteColaboradors (int id) {
		String[] args = new String[]{Integer.toString(id)};
		myDataBase.delete(taulaColaboradorsProjecte, "idProjecte = ?", args);
		return true;
	}
	
	/**
	 * Delete contacte.
	 *
	 * @param id the id
	 * @return the boolean
	 */
	public Boolean deleteContacte (int id) {
		String[] args = new String[]{Integer.toString(id)};
		myDataBase.delete(taulaContactes, "idContacte = ?", args);
		return true;
	}

	/**
	 * Gets the ID contacte.
	 *
	 * @param nom the nom
	 * @return the ID contacte
	 */
	public int getIDContacte (String nom) {
		int id;
		String[] columnes = new String[] {idContacte};
		
		Cursor resultat = myDataBase.query(taulaContactes, columnes, nomContacte + " = '" + nom 
				+ "'", null, null, null, null);
		
		if (resultat.moveToFirst()) {
			id = resultat.getInt(resultat.getColumnIndex(idContacte));
			return id;
		}else {
			return -1;
		}
	}
	
	/**
	 * Gets the ID contacte.
	 *
	 * @param nomCon the nom con
	 * @param tipusCon the tipus con
	 * @param nif the nif
	 * @param carrerCon the carrer con
	 * @param ciutatCon the ciutat con
	 * @param codiPostalCon the codi postal con
	 * @param paisCon the pais con
	 * @param emailCon the email con
	 * @param telefonCon the telefon con
	 * @param faxCon the fax con
	 * @param urlCon the url con
	 * @return the ID contacte
	 */
	public int getIDContacte (String nomCon, int tipusCon, String nif, String carrerCon, String ciutatCon,
			String codiPostalCon, String paisCon, String emailCon, String telefonCon, String faxCon, String urlCon) {
		int id;
		String[] columnes = new String[] {idContacte};
		
		Cursor resultat = myDataBase.query(taulaContactes, columnes, 
				nomContacte + " = '" + nomCon + "' AND "
				+ tipusContacte + " = " + tipusCon + " AND "
				+ nifCif + " = '" + nif + "' AND "
				+ carrer + " = '" + carrerCon + "' AND "
				+ ciutat + " = '" + ciutatCon + "' AND " 
				+ codiPostal + " = '" + codiPostalCon + "' AND "
				+ pais + " = '" + paisCon + "' AND " 
				+ email + " = '" + emailCon + "' AND "
				+ telefon + " = '" + telefonCon + "' AND " 
				+ fax + " = '" + faxCon + "' AND "
				+ urlWeb + " = '" + urlCon + "'"
				, null, null, null, null);
		
		if (resultat.moveToFirst()) {
			id = resultat.getInt(resultat.getColumnIndex(idContacte));
			return id;
		}else {
			return -1;
		}
	}


	
	/*
	* PRESSUPOSTOS I PROJECTES
	**/
	
	/**
	 * Gets the projecte.
	 *
	 * @param id the id
	 * @return the projecte
	 * @throws ParseException the parse exception
	 */
	public Projecte getProjecte (int id) throws ParseException {
		Projecte projecte;
		Factura factura;
		Contacte client;
		Tarifa tarifa;
		Concepte concepte;
		ArrayList<PreuUnitariFac> preusUnitarisFac;
		ArrayList<ImpostFac> impostosFac;
		ArrayList<VariableFac> variablesFac;
		ArrayList<Contacte> colaboradors;
		Date dataEnt,dataCre;
		float importPro;
		Cursor resultat;
		
		String[] columnes = new String[] {idProjecte, nomProjecte, idFactura, idContacte, descClient, idTarifa, idConcepte, 
				dataEntrega, dataCreacio, estatProjecte, observacions, importProjecte};
		
		resultat = myDataBase.query(taulaProjectes, columnes, idProjecte + " = " + id, null, null, null, null);
		
		if (resultat.moveToFirst()) {
			factura = getFactura(resultat.getString(resultat.getColumnIndex(idFactura)));
			client = getContacte(resultat.getInt(resultat.getColumnIndex(idContacte)));
			tarifa = getTarifa(resultat.getInt(resultat.getColumnIndex(idTarifa)));
			concepte = getConcepte(resultat.getInt(resultat.getColumnIndex(idConcepte)));
			preusUnitarisFac = getPreuUnitariFac(resultat.getColumnIndex(idProjecte));
			impostosFac = getImpostFac(resultat.getColumnIndex(idProjecte));
			variablesFac = getVariableFac(resultat.getColumnIndex(idProjecte));
			colaboradors = getColaboradors(resultat.getColumnIndex(idProjecte));
			
			dataEnt = format.parse(resultat.getString(resultat.getColumnIndex(dataEntrega)));
			dataCre = format.parse(resultat.getString(resultat.getColumnIndex(dataCreacio)));
			importPro = Float.parseFloat(resultat.getString(resultat.getColumnIndex(importProjecte)));
		
			projecte = new Projecte(
					resultat.getInt(resultat.getColumnIndex(idProjecte)),
					resultat.getString(resultat.getColumnIndex(nomProjecte)),
					client,
					resultat.getString(resultat.getColumnIndex(descClient)),
					concepte,
					tarifa,
					preusUnitarisFac,
					dataEnt,
					dataCre,
					impostosFac,
					variablesFac,
					resultat.getInt(resultat.getColumnIndex(estatProjecte)),
					colaboradors,
					resultat.getString(resultat.getColumnIndex(observacions)),
					importPro,
					factura);
			return projecte;
		}else {
			return null;
		}
	}
	
	/**
	 * Gets the projectes.
	 *
	 * @param tipus the tipus
	 * @return the projectes
	 * @throws ParseException the parse exception
	 */
	public ArrayList<Projecte> getProjectes(int tipus) throws ParseException {
		Factura factura;
		Contacte client;
		Tarifa tarifa;
		Concepte concepte;
		ArrayList<PreuUnitariFac> preusUnitarisFac;
		ArrayList<ImpostFac> impostosFac;
		ArrayList<VariableFac> variablesFac;
		ArrayList<Contacte> colaboradors;
		Date dataEnt,dataCre;
		int idP;
		float importPro;
		Cursor resultat;
		
		ArrayList<Projecte> projectes = new ArrayList<Projecte>();
		
		String[] columnes = new String[] {idProjecte, nomProjecte, idFactura, idContacte, descClient, idTarifa, idConcepte, 
				dataEntrega, dataCreacio, estatProjecte, observacions, importProjecte};
		
		switch (tipus) {
			case PRESSUPOST:
				resultat = myDataBase.query(taulaProjectes, columnes, estatProjecte + " = " + PRESSUPOST, null, null, null, dataEntrega);
				break;
			case PROJECTE:
				resultat = myDataBase.query(taulaProjectes, columnes, estatProjecte + " <> " + PRESSUPOST, null, null, null, dataEntrega);
				break;
			case FACTURA:
			resultat = myDataBase.query(taulaProjectes, columnes, idFactura
					+ " IS NOT NULL", null, null, null, dataEntrega);
				break;
			default:
				resultat = null;
		}
		
		if (resultat.moveToFirst()) {
			do {
				idP = resultat.getInt(resultat.getColumnIndex(idProjecte));
				factura = getFactura(resultat.getString(resultat.getColumnIndex(idFactura)));
				client = getContacte(resultat.getInt(resultat.getColumnIndex(idContacte)));
				tarifa = getTarifa(resultat.getInt(resultat.getColumnIndex(idTarifa)));
				concepte = getConcepte(resultat.getInt(resultat.getColumnIndex(idConcepte)));
				preusUnitarisFac = getPreuUnitariFac(idP);
				impostosFac = getImpostFac(idP);
				variablesFac = getVariableFac(idP);
				colaboradors = getColaboradors(idP);
				
				dataEnt = format.parse(resultat.getString(resultat.getColumnIndex(dataEntrega)));
				dataCre = format.parse(resultat.getString(resultat.getColumnIndex(dataCreacio)));
				importPro = Float.parseFloat(resultat.getString(resultat.getColumnIndex(importProjecte)));
				
				projectes.add(new Projecte(
						idP,
						resultat.getString(resultat.getColumnIndex(nomProjecte)),
						client,
						resultat.getString(resultat.getColumnIndex(descClient)),
						concepte,
						tarifa,
						preusUnitarisFac,
						dataEnt,
						dataCre,
						impostosFac,
						variablesFac,
						resultat.getInt(resultat.getColumnIndex(estatProjecte)),
						colaboradors,
						resultat.getString(resultat.getColumnIndex(observacions)),
						importPro,
						factura));
			} while(resultat.moveToNext());
		}
		return projectes;
	}
	
	/**
	 * Insert projecte.
	 *
	 * @param nomPro the nom pro
	 * @param idFact the id fact
	 * @param idCont the id cont
	 * @param descCli the desc cli
	 * @param idTar the id tar
	 * @param idConc the id conc
	 * @param dataEnt the data ent
	 * @param dataCre the data cre
	 * @param estatProj the estat proj
	 * @param obs the obs
	 * @param importProj the import proj
	 * @return the int
	 */
	public int insertProjecte (String nomPro, String idFact, int idCont, String descCli, int idTar, int idConc, Date dataEnt,
			Date dataCre, int estatProj, String obs, String importProj) {
		
		ContentValues valors = new ContentValues();
		valors.put(nomProjecte, nomPro);
		valors.put(idFactura, idFact);
		valors.put(idContacte, idCont);
		valors.put(descClient, descCli);
		valors.put(idTarifa, idTar);
		valors.put(idConcepte, idConc);
		valors.put(dataEntrega, format.format(dataEnt));
		valors.put(dataCreacio, format.format(dataCre));
		valors.put(estatProjecte, estatProj);
		valors.put(observacions, obs);
		valors.put(importProjecte, importProj);
				
		return (int) myDataBase.insert(taulaProjectes,null,valors);
	}
	
	/**
	 * Update projecte.
	 *
	 * @param idPro the id pro
	 * @param nomPro the nom pro
	 * @param idFact the id fact
	 * @param idCont the id cont
	 * @param descCli the desc cli
	 * @param idTar the id tar
	 * @param idConc the id conc
	 * @param dataEnt the data ent
	 * @param dataCre the data cre
	 * @param estatProj the estat proj
	 * @param obs the obs
	 * @param importProj the import proj
	 * @return the boolean
	 */
	public Boolean updateProjecte (int idPro, String nomPro, String idFact, int idCont, String descCli, int idTar, int idConc, Date dataEnt,
			Date dataCre, int estatProj, String obs, String importProj) {
		
		ContentValues valors = new ContentValues();
		valors.put(nomProjecte, nomPro);
		valors.put(idFactura, idFact);
		valors.put(idContacte, idCont);
		valors.put(descClient, descCli);
		valors.put(idTarifa, idTar);
		valors.put(idConcepte, idConc);
		valors.put(dataEntrega, format.format(dataEnt));
		valors.put(dataCreacio, format.format(dataCre));
		valors.put(estatProjecte, estatProj);
		valors.put(observacions, obs);
		valors.put(importProjecte, importProj);
		 
		String[] args = new String[]{Integer.toString(idPro)};
		myDataBase.update(taulaProjectes, valors, "idProjecte = ?", args);
		return true;
	}
	
	/**
	 * Associar client.
	 *
	 * @param idCli the id cli
	 * @param idPro the id pro
	 * @return the boolean
	 */
	public Boolean associarClient (int idCli, int idPro) {
		
		ContentValues valors = new ContentValues();
		valors.put(idContacte, idCli);
		 
		String[] args = new String[]{Integer.toString(idPro)};
		myDataBase.update(taulaProjectes, valors, "idProjecte = ?", args);
		return true;
	}
	
	/**
	 * Acceptar pressupost.
	 *
	 * @param idPro the id pro
	 * @param dataCre the data cre
	 * @return the boolean
	 */
	public Boolean acceptarPressupost (int idPro, String dataCre) {
		
		ContentValues valors = new ContentValues();
		valors.put(estatProjecte, 2);
		valors.put(dataCreacio, dataCre);
		 
		String[] args = new String[]{Integer.toString(idPro)};
		if(myDataBase.update(taulaProjectes, valors, "idProjecte = ?", args) == 1) {
			return true;
		}else return false;
	}
	
	/**
	 * Modificar estat projecte.
	 *
	 * @param idPro the id pro
	 * @param dataEnt the data ent
	 * @param estat the estat
	 * @return the boolean
	 */
	public Boolean modificarEstatProjecte (int idPro, String dataEnt, int estat) {
		
		ContentValues valors = new ContentValues();
		valors.put(estatProjecte, estat);
		valors.put(dataEntrega, dataEnt);
		 
		String[] args = new String[]{Integer.toString(idPro)};
		if(myDataBase.update(taulaProjectes, valors, "idProjecte = ?", args) == 1) {
			return true;
		}else return false;
	}
	
	/**
	 * Modificar estat projecte.
	 *
	 * @param idPro the id pro
	 * @param estat the estat
	 * @return the boolean
	 */
	public Boolean modificarEstatProjecte (int idPro, int estat) {
		
		ContentValues valors = new ContentValues();
		valors.put(estatProjecte, estat);
		 
		String[] args = new String[]{Integer.toString(idPro)};
		if(myDataBase.update(taulaProjectes, valors, "idProjecte = ?", args) == 1) {
			return true;
		}else return false;
	}
	
	/**
	 * Delete projecte.
	 *
	 * @param id the id
	 * @return the boolean
	 */
	public Boolean deleteProjecte (int id) {
		String[] args = new String[]{Integer.toString(id)};
		myDataBase.delete(taulaProjectes, "idProjecte = ?", args);
		return true;
	}

	/**
	 * Gets the ID projecte.
	 *
	 * @param nom the nom
	 * @return the ID projecte
	 */
	public int getIDProjecte (String nom) {
		int id;
		String[] columnes = new String[] {idProjecte};
		
		Cursor resultat = myDataBase.query(taulaProjectes, columnes, nomProjecte + " = '" + nom 
				+ "'", null, null, null, null);
		
		if (resultat.moveToFirst()) {
			id = resultat.getInt(resultat.getColumnIndex(idProjecte));
			return id;
		}else {
			return -1;
		}
	}
	
	/**
	 * Gets the ID projecte.
	 *
	 * @param nomPro the nom pro
	 * @param idFact the id fact
	 * @param idCont the id cont
	 * @param descCli the desc cli
	 * @param idTar the id tar
	 * @param idConc the id conc
	 * @param dataEnt the data ent
	 * @param dataCre the data cre
	 * @param estatProj the estat proj
	 * @param obs the obs
	 * @param importProj the import proj
	 * @return the ID projecte
	 */
	public int getIDProjecte (String nomPro, String idFact, int idCont, String descCli, int idTar, int idConc, Date dataEnt,
			Date dataCre, int estatProj, String obs, String importProj) {
		String[] columnes = new String[] {idProjecte};
		
		Cursor resultat = myDataBase.query(taulaProjectes, columnes, 
				nomProjecte + " = '" + nomPro + "' AND "
				+ idFactura + " = '" + idFact + "' AND "
				+ idContacte + " = " + idCont + " AND "
				+ descClient + " = '" + descCli + "' AND "
				+ idTarifa + " = " + idTar + " AND " 
				+ idConcepte + " = " + idConc + " AND "
				+ dataEntrega + " = '" + format.format(dataEnt) + "' AND " 
				+ dataCreacio + " = '" + format.format(dataCre) + "' AND "
				+ estatProjecte + " = " + estatProj + " AND " 
				+ observacions + " = '" + obs + "' AND "
				+ importProjecte + " = '" + importProj + "'"
				, null, null, null, null);
		
		if (resultat.moveToFirst()) {
			return resultat.getInt(resultat.getColumnIndex(idProjecte));
		}else {
			return -1;
		}
	}
	
	
	/*
	 * FACTURES
	 * */
	
	/**
	 * Gets the factura.
	 *
	 * @param id the id
	 * @return the factura
	 * @throws ParseException the parse exception
	 */
	public Factura getFactura (String id) throws ParseException {
		Factura factura;
		Contacte client;
		Date dataCob,dataEst;
		float importFac;
		
		String[] columnes = new String[] {identificadorFactura, idContacte, dataCobrament, estatFactura, dataEstat
				, importFactura};
		
		Cursor resultat = 
				myDataBase.query(taulaFactures, columnes, identificadorFactura + " = '" + id + "'", null, null, null, null);
		
		if (resultat.moveToFirst()) {
			client = getContacte(resultat.getInt(resultat.getColumnIndex(idContacte)));
			dataCob = format.parse(resultat.getString(resultat.getColumnIndex(dataCobrament)));
			dataEst = format.parse(resultat.getString(resultat.getColumnIndex(dataEstat)));
			importFac = Float.parseFloat(resultat.getString(resultat.getColumnIndex(importFactura)));
			
			factura = new Factura(
					resultat.getString(resultat.getColumnIndex(identificadorFactura)),
					client,
					dataCob,
					resultat.getInt(resultat.getColumnIndex(estatFactura)),
					dataEst,
					importFac);
			return factura;
		}else {
			return null;
		}
	}
	
	/**
	 * Insert factura.
	 *
	 * @param idPro the id pro
	 * @param idFac the id fac
	 * @param idCont the id cont
	 * @param dataCob the data cob
	 * @param est the est
	 * @param dataEst the data est
	 * @param impTotal the imp total
	 * @return the int
	 */
	public int insertFactura (String idPro, String idFac, int idCont, Date dataCob, int est, Date dataEst, float impTotal) {
		
		ContentValues valorsFactura = new ContentValues(),valorsProjecte = new ContentValues() ;
		
		valorsProjecte.put(idFactura, idFac);
		String[] args = new String[]{idPro};
		myDataBase.update(taulaProjectes, valorsProjecte, "idProjecte = ?", args);
		
		valorsFactura.put(identificadorFactura, idFac);
		valorsFactura.put(idContacte, idCont);
		valorsFactura.put(dataCobrament, format.format(dataCob));
		valorsFactura.put(estatFactura, est);
		valorsFactura.put(dataEstat, format.format(dataEst));
		valorsFactura.put(importFactura, impTotal);
		
		return (int) myDataBase.insert(taulaFactures,null,valorsFactura);
	}
	
	public Boolean deleteFactura(String id) {
		String[] args = new String[] { id };
		boolean deleted = false;

		ContentValues valors = new ContentValues();
		valors.put(idFactura, (String) null);

		try {
			if (myDataBase
					.update(taulaProjectes, valors, "idFactura = ?", args) == 1) {
				if (myDataBase.delete(taulaFactures,
						"identificadorFactura = ?", args) > 0)
					deleted = true;
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return deleted;
	}
	
	public Boolean modificarEstatFactura(String idFac, int estat) {

		ContentValues valors = new ContentValues();
		valors.put(estatFactura, estat);

		String[] args = new String[] { idFac };
		if (myDataBase.update(taulaFactures, valors,
				"identificadorFactura = ?", args) == 1) {
			return true;
		} else
			return false;
	}

	public Boolean modificarDataFactura(String idFac, Date data) {

		ContentValues valors = new ContentValues();
		valors.put(dataCobrament, format.format(data));

		String[] args = new String[] { idFac };
		if (myDataBase.update(taulaFactures, valors,
				"identificadorFactura = ?", args) == 1) {
			return true;
		} else
			return false;
	}

	/*
	 * PREUSUNITARISFAC
	 * */
	
	/**
	 * Gets the preu unitari fac.
	 *
	 * @param idP the id p
	 * @return the preu unitari fac
	 */
	public ArrayList<PreuUnitariFac> getPreuUnitariFac(int idP) {
		PreuUnitari preuUnitari;
		ArrayList<PreuUnitariFac> preuUnitariFac = new ArrayList<PreuUnitariFac>();
		String[] columnes = new String[] {idProjecte, idPreuUnitari, numUnitats, importPreuUnitats};
		Cursor resultat = 
				myDataBase.query(taulaPreusUnitarisFac, columnes, idProjecte + " = " + idP, null, null, null, null);
		
		if (resultat.moveToFirst()) {
			do {
				preuUnitari = getPreuUnitari(resultat.getInt(resultat.getColumnIndex(idPreuUnitari)));
				
				preuUnitariFac.add(new PreuUnitariFac(preuUnitari.getNom(),
						preuUnitari.getIdPreuUnitari(),
						preuUnitari.getTarifa(),
						preuUnitari.getPreu(),
						preuUnitari.getUnitat(),
						resultat.getInt(resultat.getColumnIndex(numUnitats)),
						Float.parseFloat(resultat.getString(resultat.getColumnIndex(importPreuUnitats)))));
			} while(resultat.moveToNext());
		}
		
		return preuUnitariFac;
	}

	/**
	 * Insert preu unitari fac.
	 *
	 * @param idPro the id pro
	 * @param idPU the id pu
	 * @param uni the uni
	 * @param importPF the import pf
	 * @return the boolean
	 */
	public int insertPreuUnitariFac(int idPro, int idPU, int uni, float importPF) {

		ContentValues valors = new ContentValues();
		valors.put(idProjecte, idPro);
		valors.put(idPreuUnitari, idPU);
		valors.put(numUnitats, uni);
		valors.put(importPreuUnitats, importPF);

		return (int) myDataBase.insert(taulaPreusUnitarisFac, null, valors);
	}

	/**
	 * Delete preus unitaris fac.
	 *
	 * @param idPro the id pro
	 * @return the boolean
	 */
	public Boolean deletePreusUnitarisFac (int idPro) {
		String[] args = new String[]{Integer.toString(idPro)};
		boolean deleted = false;
		
		if (getPreuUnitariFac(idPro).isEmpty()){
			deleted = true;
		}else if (myDataBase.delete(taulaPreusUnitarisFac, "idProjecte = ?", args) > 0) {
			deleted = true;
		}
		return deleted;
	}
	
	
	/*
	 * IMPOSTOSFAC
	 * */

	/**
	 * Gets the impost fac.
	 *
	 * @param idP the id p
	 * @return the impost fac
	 * @throws ParseException the parse exception
	 */
	public ArrayList<ImpostFac> getImpostFac(int idP) throws ParseException {
		Impost impost;
		ArrayList<ImpostFac> impostosFac = new ArrayList<ImpostFac>();
		String[] columnes = new String[] {idProjecte, idImpost, importImpost};
		Cursor resultat = 
				myDataBase.query(taulaImpostosFac, columnes, idProjecte + " = " + idP, null, null, null, null);
		
		if (resultat.moveToFirst()) {
			do {
				impost = getImpost(resultat.getInt(resultat.getColumnIndex(idImpost)));
				
				impostosFac.add(new ImpostFac(
						impost.getNom(),
						impost.getIdImpost(),
						impost.getPercentatge(),
						Float.parseFloat(resultat.getString(resultat.getColumnIndex(importImpost)))));
			} while(resultat.moveToNext());
		}
		return impostosFac;
	}
	
	/**
	 * Insert impost fac.
	 *
	 * @param idPro the id pro
	 * @param idImp the id imp
	 * @param importImp the import imp
	 * @return the boolean
	 */
	public Boolean insertImpostFac (int idPro, int idImp, float importImp) {
		String[] args = new String[]{Integer.toString(idPro), Integer.toString(idImp), Float.toString(importImp)};		
		myDataBase.execSQL("INSERT INTO " + taulaImpostosFac +" (" 
				+ idProjecte + ", " 
				+ idImpost + ", " 
				+ importImpost + ") VALUES (?,?,?) ", args);
		return true;
	}
	
	/**
	 * Delete impostos fac.
	 *
	 * @param idPro the id pro
	 * @return the boolean
	 */
	public Boolean deleteImpostosFac (int idPro) {
		String[] args = new String[]{Integer.toString(idPro)};
		boolean deleted = false;
		
		try {
			if (getImpostFac(idPro).isEmpty()){
				deleted = true;
			}else if (myDataBase.delete(taulaImpostosFac, "idProjecte = ?", args) > 0) {
				deleted = true;
			}
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return deleted;
	}
	
	
	/*
	 * VARIABLESFAC
	 * */

	/**
	 * Gets the variable fac.
	 *
	 * @param idP the id p
	 * @return the variable fac
	 * @throws ParseException the parse exception
	 */
	public ArrayList<VariableFac> getVariableFac(int idP) throws ParseException {
		Variable variable;
		ArrayList<VariableFac> variablesFac = new ArrayList<VariableFac>();
		String[] columnes = new String[] {idProjecte, idVariable, importVariable};
		Cursor resultat = 
				myDataBase.query(taulaVariablesFac, columnes, idProjecte + " = " + idP, null, null, null, null);
		
		if (resultat.moveToFirst()) {
			do {
				variable = getVariable(resultat.getInt(resultat.getColumnIndex(idVariable)));
				
				variablesFac.add(new VariableFac(
						variable.getIdVariable(),
						variable.getNom(),
						variable.getValor(),
						variable.getTipusOperacio(),
						Float.parseFloat(resultat.getString(resultat.getColumnIndex(importVariable)))));
			} while(resultat.moveToNext());
		}
		return variablesFac;
	}

	/**
	 * Insert variable fac.
	 *
	 * @param idPro the id pro
	 * @param idVar the id var
	 * @param importVar the import var
	 * @return the boolean
	 */
	public Boolean insertVariableFac (int idPro, int idVar, float importVar) {
		String[] args = new String[]{Integer.toString(idPro), Integer.toString(idVar), Float.toString(importVar)};		
		myDataBase.execSQL("INSERT INTO " + taulaVariablesFac +" (" 
				+ idProjecte + ", " 
				+ idVariable + ", " 
				+ importVariable + ") VALUES (?,?,?) ", args);
		return true;
	}
	
	/**
	 * Delete variables fac.
	 *
	 * @param idPro the id pro
	 * @return the boolean
	 */
	public Boolean deleteVariablesFac (int idPro) {
		String[] args = new String[]{Integer.toString(idPro)};
		boolean deleted = false;
		
		try {
			if (getVariableFac(idPro).isEmpty()){
				deleted = true;
			}else if (myDataBase.delete(taulaVariablesFac, "idProjecte = ?", args) > 0) {
				deleted = true;
			}
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return deleted;
	}
}
