/*
 * Translator Helper (TH)
 * Fco. Javier Domínguez Carral
 * TFC - UOC (2014)
 */
package com.ADApps.translatorhelper.parametres.variables;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.widget.TextView;

import com.ADApps.translatorhelper.R;
import com.ADApps.translatorhelper.main.MainParametres;
import com.ADApps.translatorhelper.main.THDataBaseHelper;
import com.actionbarsherlock.app.SherlockListFragment;
import com.actionbarsherlock.view.Menu;
import com.actionbarsherlock.view.MenuInflater;
import com.actionbarsherlock.view.MenuItem;


/**
 * The Class LlistaVariablesFragment.
 */
public class LlistaVariablesFragment extends SherlockListFragment{
	
	/** The db. */
	private static THDataBaseHelper db;
	
	/** The list items. */
	private Variable[] listItems;
	
	/** The Constant TIPUS_PERCENTATGE. */
	private static final int TIPUS_PERCENTATGE = 1;
	
	/** The Constant TIPUS_SUMA. */
	private static final int TIPUS_SUMA = 2;

	/* (non-Javadoc)
	 * @see android.support.v4.app.Fragment#onCreate(android.os.Bundle)
	 */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setHasOptionsMenu(true);
		db = new THDataBaseHelper(getActivity());
		db.open();
		
		listItems = db.getVariables().toArray(new Variable[0]);
	}
	
	/* (non-Javadoc)
	 * @see android.support.v4.app.ListFragment#onCreateView(android.view.LayoutInflater, android.view.ViewGroup, android.os.Bundle)
	 */
	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container,
			Bundle savedInstanceState) {
		return inflater.inflate(R.layout.fragment_llista, container, false);
	}

	/* (non-Javadoc)
	 * @see android.support.v4.app.Fragment#onActivityCreated(android.os.Bundle)
	 */
	@Override
	public void onActivityCreated(Bundle savedInstanceState) {
		super.onActivityCreated(savedInstanceState);
		
		setListAdapter(new AdaptadorVariables(this));
		
		if (listItems.length == 0)
			((TextView) getView().findViewById(android.R.id.empty)).setText("No hi ha variables per mostrar.");
	}
	
	/**
	 * The Class AdaptadorVariables.
	 */
	class AdaptadorVariables extends ArrayAdapter<Variable> {
		
		/** The context. */
		Activity context;
		
		/**
		 * Instantiates a new adaptador variables.
		 *
		 * @param context the context
		 */
		AdaptadorVariables (SherlockListFragment context) {
			super(context.getActivity(), R.layout.listitem_variable, listItems);
			this.context = context.getActivity();
		}
		
		/* (non-Javadoc)
		 * @see android.widget.ArrayAdapter#getView(int, android.view.View, android.view.ViewGroup)
		 */
		@Override
		public View getView(int position, View convertView, ViewGroup parent) {
			String valor = new String();
			LayoutInflater inflater = context.getLayoutInflater();
			View item = inflater.inflate(R.layout.listitem_variable, null);
			
			TextView lblNomLIVariable = (TextView)item.findViewById(R.id.lblNomLIVariable);
			lblNomLIVariable.setText(listItems[position].getNom());
					
			switch (listItems[position].getTipusOperacio()) {
				case TIPUS_PERCENTATGE:
					valor = listItems[position].getValor() + "%";
					break;
				case TIPUS_SUMA:
				valor = listItems[position].getValor() + " €";
					break;
				default:
					break;
			}
			TextView lblValorLIVariable = (TextView)item.findViewById(R.id.lblValorLIVariable);
			lblValorLIVariable.setText(valor);
			
			return item;
		}
	}

	
	/* (non-Javadoc)
	 * @see android.support.v4.app.ListFragment#onListItemClick(android.widget.ListView, android.view.View, int, long)
	 */
	@Override
	public void onListItemClick(ListView l, View v, int position, long id) {
		Variable registre = (Variable) getListAdapter().getItem(position);
		
		Intent intent = new Intent(getActivity().getBaseContext(), DetallsVariableActivity.class);
		intent.putExtra("afegir", false);
		intent.putExtra("registre", registre);
		startActivity(intent);
	}
	
	/* (non-Javadoc)
	 * @see com.actionbarsherlock.app.SherlockListFragment#onCreateOptionsMenu(android.view.Menu, android.view.MenuInflater)
	 */
	@Override
	public void onCreateOptionsMenu(Menu menu, MenuInflater inflater) {
		inflater.inflate(R.menu.llista_parametres, menu);
	}
	
	/* (non-Javadoc)
	 * @see com.actionbarsherlock.app.SherlockListFragment#onOptionsItemSelected(android.view.MenuItem)
	 */
	@Override
    public boolean onOptionsItemSelected(MenuItem item) {
		Intent intent;
		switch (item.getItemId()) {
			case android.R.id.home:
				db.close();
	            intent = new Intent(getActivity(), MainParametres.class);
	            intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP|Intent.FLAG_ACTIVITY_NEW_TASK);
	            startActivity(intent);
	            return true;
			case R.id.menuLstParametresAfegir:
				db.close();
				intent = new Intent(getActivity().getBaseContext(), DetallsVariableActivity.class);
				intent.putExtra("afegir", true);
				startActivity(intent);
	            return true;
			default:
				return super.onOptionsItemSelected(item);
        }
    }
}

