/*
 * Translator Helper (TH)
 * Fco. Javier Domínguez Carral
 * TFC - UOC (2014)
 */
package com.ADApps.translatorhelper.pressupostos;

import java.text.ParseException;
import java.text.SimpleDateFormat;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.widget.TextView;

import com.ADApps.translatorhelper.main.*;
import com.ADApps.translatorhelper.projectes.Projecte;
import com.ADApps.translatorhelper.R;
import com.actionbarsherlock.app.SherlockListFragment;
import com.actionbarsherlock.view.Menu;
import com.actionbarsherlock.view.MenuInflater;
import com.actionbarsherlock.view.MenuItem;


/**
 * The Class LlistaPressupostosFragment.
 */
public class LlistaPressupostosFragment extends SherlockListFragment{
	
	/** The Constant PRESSUPOST. */
	private static final int PRESSUPOST = 1;
	
	/** The db. */
	private static THDataBaseHelper db;
	
	/** The list items. */
	private Projecte[] listItems;

	/* (non-Javadoc)
	 * @see android.support.v4.app.Fragment#onCreate(android.os.Bundle)
	 */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setHasOptionsMenu(true);
		db = new THDataBaseHelper(getActivity());
		db.open();
		
		try {
			listItems = (Projecte[]) db.getProjectes(PRESSUPOST).toArray(new Projecte[0]);
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/* (non-Javadoc)
	 * @see android.support.v4.app.ListFragment#onCreateView(android.view.LayoutInflater, android.view.ViewGroup, android.os.Bundle)
	 */
	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container,
			Bundle savedInstanceState) {
		return inflater.inflate(R.layout.fragment_llista, container, false);
	}
	
	/* (non-Javadoc)
	 * @see android.support.v4.app.Fragment#onActivityCreated(android.os.Bundle)
	 */
	@Override
	public void onActivityCreated(Bundle savedInstanceState) {
		super.onActivityCreated(savedInstanceState);
		
		setListAdapter(new AdaptadorPressupostos(this));
		
		if (listItems.length == 0)
			((TextView) getView().findViewById(android.R.id.empty)).setText("No hi ha pressupostos per mostrar.");
	}
	
	/**
	 * The Class AdaptadorPressupostos.
	 */
	class AdaptadorPressupostos extends ArrayAdapter<Projecte> {
		
		/** The context. */
		Activity context;
		
		/**
		 * Instantiates a new adaptador pressupostos.
		 *
		 * @param context the context
		 */
		AdaptadorPressupostos (SherlockListFragment context) {
			super(context.getActivity(), R.layout.listitem_pressupost, listItems);
			this.context = context.getActivity();
		}
		
		/* (non-Javadoc)
		 * @see android.widget.ArrayAdapter#getView(int, android.view.View, android.view.ViewGroup)
		 */
		public View getView(int position, View convertView, ViewGroup parent) {
			LayoutInflater inflater = context.getLayoutInflater();
			View item = inflater.inflate(R.layout.listitem_pressupost, null);
			
			TextView lblClientLIPressupost = (TextView)item.findViewById(R.id.lblClientLIPressupost);
			if (listItems[position].getClient() == null){
				lblClientLIPressupost.setText(listItems[position].getDescClient());
			}else{
				lblClientLIPressupost.setText(listItems[position].getClient().getNomCognoms());
			}
			
			
			TextView lblNomLIPressupost = (TextView)item.findViewById(R.id.lblNomLIPressupost);
			lblNomLIPressupost.setText(listItems[position].getNomProjecte());
			
			TextView lblImportLIPressupost = (TextView)item.findViewById(R.id.lblImportLIPressupost);
			lblImportLIPressupost.setText(Float.toString(listItems[position].getImportProjecte()));
			
			TextView lblDataLIPressupost = (TextView)item.findViewById(R.id.lblDataLIPressupost);
			
			SimpleDateFormat sdf = new SimpleDateFormat ("dd/MM/yyyy");
			lblDataLIPressupost.setText(sdf.format(listItems[position].getDataCreacio()));
			
			return item;
		}
	}
	
	/* (non-Javadoc)
	 * @see android.support.v4.app.ListFragment#onListItemClick(android.widget.ListView, android.view.View, int, long)
	 */
	@Override
	public void onListItemClick(ListView l, View v, int position, long id) {
		Projecte registre = (Projecte) getListAdapter().getItem(position);
		
		Intent intent = new Intent(getActivity().getBaseContext(), DetallsPressupostActivity.class);
		intent.putExtra("afegir", false);
		intent.putExtra("registre", registre);
		startActivity(intent);
	}
	
	/* (non-Javadoc)
	 * @see com.actionbarsherlock.app.SherlockListFragment#onCreateOptionsMenu(android.view.Menu, android.view.MenuInflater)
	 */
	@Override
	public void onCreateOptionsMenu(Menu menu, MenuInflater inflater) {
		inflater.inflate(R.menu.llista_parametres, menu);
	}
	
	/* (non-Javadoc)
	 * @see com.actionbarsherlock.app.SherlockListFragment#onOptionsItemSelected(android.view.MenuItem)
	 */
	@Override
    public boolean onOptionsItemSelected(MenuItem item) {
		Intent intent;
		switch (item.getItemId()) {
			case android.R.id.home:
				db.close();
	            intent = new Intent(getActivity(), MainActivity.class);
	            intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP|Intent.FLAG_ACTIVITY_NEW_TASK);
	            startActivity(intent);
	            return true;
			case R.id.menuLstParametresAfegir:
				db.close();
				intent = new Intent(getActivity().getBaseContext(), DetallsPressupostActivity.class);
				intent.putExtra("afegir", true);
				startActivity(intent);
	            return true;
			default:
				return super.onOptionsItemSelected(item);
        }
    }	
}
