/*
 * Translator Helper (TH)
 * Fco. Javier Domínguez Carral
 * TFC - UOC (2014)
 */
package com.ADApps.translatorhelper.projectes;

import java.text.ParseException;
import java.text.SimpleDateFormat;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.widget.TextView;

import com.ADApps.translatorhelper.R;
import com.ADApps.translatorhelper.main.MainActivity;
import com.ADApps.translatorhelper.main.THDataBaseHelper;
import com.actionbarsherlock.app.SherlockListFragment;
import com.actionbarsherlock.view.MenuItem;


/**
 * The Class LlistaProjectesFragment.
 */
public class LlistaProjectesFragment extends SherlockListFragment{
	
	/** The Constant PROJECTE. */
	private static final int PROJECTE = 2;
	
	/** The Constant EN_CURS. */
	private static final int EN_CURS = 2;
	
	/** The Constant ENTREGAT. */
	private static final int ENTREGAT = 3;
	
	/** The db. */
	private static THDataBaseHelper db;
	
	/** The list items. */
	private Projecte[] listItems;
	
	/* (non-Javadoc)
	 * @see android.support.v4.app.Fragment#onCreate(android.os.Bundle)
	 */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setHasOptionsMenu(true);
		db = new THDataBaseHelper(getActivity());
		db.open();
		
		try {
			listItems = (Projecte[]) db.getProjectes(PROJECTE).toArray(new Projecte[0]);
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	/* (non-Javadoc)
	 * @see android.support.v4.app.ListFragment#onCreateView(android.view.LayoutInflater, android.view.ViewGroup, android.os.Bundle)
	 */
	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container,
			Bundle savedInstanceState) {
		return inflater.inflate(R.layout.fragment_llista, container, false);
	}

	/* (non-Javadoc)
	 * @see android.support.v4.app.Fragment#onActivityCreated(android.os.Bundle)
	 */
	@Override
	public void onActivityCreated(Bundle savedInstanceState) {
		super.onActivityCreated(savedInstanceState);
		
		setListAdapter(new AdaptadorProjectes(this));

		if (listItems.length == 0)
			((TextView) getView().findViewById(android.R.id.empty)).setText("No hi ha projectes per mostrar.");
	}
	
	/**
	 * The Class AdaptadorProjectes.
	 */
	class AdaptadorProjectes extends ArrayAdapter<Projecte> {
		
		/** The context. */
		Activity context;
		
		/**
		 * Instantiates a new adaptador projectes.
		 *
		 * @param context the context
		 */
		AdaptadorProjectes (SherlockListFragment context) {
			super(context.getActivity(), R.layout.listitem_projecte, listItems);
			this.context = context.getActivity();
		}
		
		/* (non-Javadoc)
		 * @see android.widget.ArrayAdapter#getView(int, android.view.View, android.view.ViewGroup)
		 */
		public View getView(int position, View convertView, ViewGroup parent) {
			SimpleDateFormat sdf = new SimpleDateFormat ("dd/MM/yyyy");
			LayoutInflater inflater = context.getLayoutInflater();
			View item = inflater.inflate(R.layout.listitem_projecte, null);
			
			TextView client = (TextView)item.findViewById(R.id.lblClientLIProjecte);
			if (listItems[position].getClient() == null){
				client.setText(listItems[position].getDescClient());
			}else{
				client.setText(listItems[position].getClient().getNomCognoms());
			}
			
			
			TextView descProjecte = (TextView)item.findViewById(R.id.lblDescripcioLIProjecte);
			descProjecte.setText(listItems[position].getNomProjecte());
			
			TextView estat = (TextView)item.findViewById(R.id.lblEstatLIProjecte);
			
			switch (listItems[position].getEstat()){
				case EN_CURS:
					estat.setText(R.string.estatPro_enCurs);
					break;
				case ENTREGAT:
					estat.setText(R.string.estatPro_entregat);
					break;
				default:
					break;
			}

			TextView dataProjecte = (TextView)item.findViewById(R.id.lblDataLIProjecte);
			dataProjecte.setText(sdf.format(listItems[position].getDataEntrega()));
			
			return item;
		}
	}
	
	/* (non-Javadoc)
	 * @see android.support.v4.app.ListFragment#onListItemClick(android.widget.ListView, android.view.View, int, long)
	 */
	@Override
	public void onListItemClick(ListView l, View v, int position, long id) {
		Projecte registre = (Projecte) getListAdapter().getItem(position);
		
		Intent intent = new Intent(getActivity().getBaseContext(), DetallsProjecteActivity.class);
		intent.putExtra("registre", registre);
		startActivity(intent);
	}
	
	/* (non-Javadoc)
	 * @see com.actionbarsherlock.app.SherlockListFragment#onOptionsItemSelected(android.view.MenuItem)
	 */
	@Override
    public boolean onOptionsItemSelected(MenuItem item) {
		Intent intent;
		switch (item.getItemId()) {
			case android.R.id.home:
				db.close();
	            intent = new Intent(getActivity(), MainActivity.class);
	            intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP|Intent.FLAG_ACTIVITY_NEW_TASK);
	            startActivity(intent);
	            return true;
			default:
				return super.onOptionsItemSelected(item);
        }
    }
}
